#ifndef _UMI_H
#define _UMI_H

/**
 * \file umi.h Universal Message Interface - C part
 */
#include "umidefs.h"
#include <stddef.h>

/* 
 * ioctls
 */
#define GIOC_PROCHK 0	/* check protocol			*/
#define GIOC_PUTMSG 1	/* put a message to the kernel		*/
#define GIOC_GETMSG 2	/* get a message from the kernel	*/
#define GIOC_EXCMSG 3	/* exchange messages with the kernel	*/


#ifdef __cplusplus
extern "C" {
#endif

  #define __SZ_DATA_MAX 512	/* UMI private		*/
  #define SZ_CTRL_MAX 128	/* including header	*/

  
  /**
   * incore rep of the message parts
   * 
   */
  struct UmiMemStruct {
    /* buffer */
    char *_pchLimBuf;	/* last byte +1 of buffer	 */
    char *_pchLimDat;	/* last byte +1 of valid data	 */
    char *_pchBase;	/* first byte of allocation	 */
  };

  /**
   * parameter set for the user/kernel boundary (IOCTL)
   */
  typedef struct {
    struct UmiMemStruct _cd;
    struct UmiMemStruct _dd;
    struct UmiMemStruct _cu;
    struct UmiMemStruct _du;
  } UmiIoc;

  /**
   * control message header
   */
  struct UmiControlStruct {

    /*
     * size and type of this
     */
    size_t    _size;	 	/* intra-machine copy  	*/
    unsigned  _eType;	 	/* message type		*/

    /* more control follows here 	*/
    /* ....		 		*/

  };

  struct timespec;

  /** 
   * result codes for putDown() of the driver
   * \note might be without any effect
   */
  #define MSG_DONE 0	/* task done			*/
  #define MSG_WAIT 1	/* task remains		   	*/
  #define MSG_BUSY 2	/* not able to create task 	*/

  /* size of the header on wire */
  #define SZ_WIRE_HEADER 1


#ifdef __cplusplus
}
#endif

#endif
