
#include "fixme.h"
#include "mod.h" 
#include "card.h" 
#include "assert.h" 
#include <linux/string.h>
#include <linux/fs.h>
#include <linux/pci.h>
#include <linux/proc_fs.h>

  /*
   * Card structure allocation/release
   */
  int pdm_allocslot(LumiMod *pdm, const char *sz) {
    int i;
    assert(sz);
    if ((i=pdm_findslot(pdm, sz)) != -1) return i;

    for (i=0; i<MAX_CARD; i++) {
      if (!strcmp(pdm->_mprc[i]._mzId,"")) {
        /* allocate the slot */
	strlcpy(pdm->_mprc[i]._mzId, sz, SZ_CARD_ID);
	return i;
      }
    }

    /* can't allocate */
    return -1;
  }

  int pdm_findslot(LumiMod *pdm, const char *sz) {
    int i;
    assert(sz);
    for (i=0; i<MAX_CARD; i++) {
      if (!strcmp(pdm->_mprc[i]._mzId,sz)) return i;
    }

    /* can't find */
    return -1;
  }


  extern struct CardStruct* pdm_setcard(LumiMod *pdm, int i, struct CardStruct *pca) {
    assert(i>=0 && i<MAX_CARD);
    assert(pca == 0 || pdm->_mprc[i]._pca==0);
    pdm->_mprc[i]._pca=pca;
    return pca;
  };

  extern struct CardStruct* pdm_getcard(LumiMod *pdm, int i) {
    assert(i>=0 && i<MAX_CARD);
    return pdm->_mprc[i]._pca;
  }

  extern struct CardStruct* pdm_clrcard(LumiMod *pdm, int i) {
    struct CardStruct * pca;
    assert(i>=0 && i<MAX_CARD);
    pca=pdm->_mprc[i]._pca;
    pdm->_mprc[i]._pca=0;
    return pca;
  }

  extern char* pdm_getid(LumiMod *pdm, struct CardStruct *pca) {
    int i;
    for (i=0; i<MAX_CARD; i++) {
      if (pdm->_mprc[i]._pca==pca) return pdm->_mprc[i]._mzId;
    }
    assert(0);
    return 0;
  }

  int pdm_getslot(LumiMod *pdm, const char *szId) {
    int iSlot;
    if ((iSlot=pdm_allocslot(pdm, szId))==-1) {
      printk(KERN_INFO PFX "slot allocation error\n",pdm->_szDid);
      return -1;
    }
    else {
      printk(KERN_INFO PFX "allocated slot %d \n",pdm->_szDid, iSlot);
    }

    if (pdm_getcard(pdm, iSlot)) {
      /* already registered */
      printk(KERN_INFO PFX "already initialized %d\n",pdm->_szDid, iSlot);
      return -1;
    }
    return iSlot;
  }

  int pdm_read_procmem( char* pchBuf, char **ppch, off_t offs , int c, int *piEof, void *p) {
    LumiMod *pdm=(LumiMod*) p;

    int i;
    char *pch=pchBuf;
    char *pchLim=pchBuf+c-80;

    for (i=0; i<MAX_CARD && pch < pchLim; i++) {
      Card *pca=pdm->_mprc[i]._pca;
      if (!strcmp(pdm->_mprc[i]._mzId,"")) break;
      pch += sprintf(pch, "adaptor %d: %s ", i, pdm->_mprc[i]._mzId);
      if (pca) {
        pch=(*pca->_pfPrint)(pca, pch, c-(pch-pchBuf));
	pch += sprintf(pch, "\n");
      }
      else {
	pch += sprintf(pch, "(unplugged)\n");
      }
    }
    *piEof=1;
    return pch-pchBuf;
  }



  /*
   * init/cleanup module
   */
  void pdm_cleanup_module(LumiMod *pdm) {
   #ifdef __DEVFS
    devfs_unregister(plm->_hDfs);
    devfs_unregister_chrdev(plm->_iMajor,plm->_szDid);
   #else
    unregister_chrdev(pdm->_iMajor,pdm->_szDid);
   #endif
    printk(KERN_INFO "%s: cleanup module, bye ...\n", pdm->_szDid);
    if (pdm->_pUdr) usb_deregister(pdm->_pUdr);
    if (pdm->_pPdr) pci_unregister_driver (pdm->_pPdr);
    free_isa_devices(pdm);

    if (pdm->_szProcDir) remove_proc_entry(pdm->_szProcDir, 0);
  }


  int pdm_init_module(LumiMod *pdm) {
    printk(KERN_INFO "%s: %s driver %s for Linux\n", pdm->_szDid, pdm->_szName,  pdm->_szDvs);

    /*
     * init the UmiMod structure
     */
    (*pdm->_pum->_pFct)();

    /*
     * enumerate ISA cards
     */
    if (enum_isa_devices(pdm)) return -ENODEV;

    /* 
     * register PCI
     */
    if (pdm->_pPdr) (void) pci_register_driver(pdm->_pPdr);
    if (pdm->_pUdr) (void) usb_register(pdm->_pUdr);


    pdm->_iMajor=0;

   #ifdef __DEVFS
    plm->_iMajor=devfs_register_chrdev(plm->_iMajor,plm->_szDid,plm->_pFops);
    if(plm->_iMajor<0) {
      printk(KERN_ERR "%s: can't get major %d\n",plm->_szDid, plm->_iMajor);
      lumi_cleanup_module(plm);
      return -ENODEV;
    }
    plm->_hDfs=devfs_mk_dir(NULL, plm->_szDid, NULL);
   #else

    pdm->_iMajor=register_chrdev(pdm->_iMajor,pdm->_szDid,pdm->_pFops);

   #endif

    if (pdm->_szProcDir) {
      if (!create_proc_read_entry(pdm->_szProcDir, 0, 0, pdm_read_procmem, pdm)) {
	printk(KERN_ERR "%s: can't create <proc>/%s\n", pdm->_szDid, pdm->_szProcDir);
	return -ENOMEM;
      }
    }

    return 0;	/* keep driver for hotplug events */
  }


