
/*
 * driver application wrapper demo
 */

#include <linux/init.h>
#include <linux/types.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/pci.h>
#include <linux/usb.h>
#include <linux/version.h>
#include "mod.h"
#include "gpibc/gumi_c.h"
#include "gpibc/arch.h"
#include "gpibc/cards.h"

int _GLOBAL_OFFSET_TABLE_;

/*
 * declare the license of this module
 */
MODULE_LICENSE("GPL");


  struct LumiModStruct dm ;		/* forward declaration */

  /* table of devices that work with this driver */
  static struct usb_device_id mpUdi [] = {
    { USB_DEVICE(0x04b4, 0x2506), .driver_info=(kernel_ulong_t)&adUsb },
    { }                                     /* Terminating entry */
  };

  /*
   * announce our capabilities 
   *
   * that's why we need the static xxx_device_id tables
   * in the wrapper module
   */
  MODULE_DEVICE_TABLE(pci, mpPdi);
  MODULE_DEVICE_TABLE(usb, mpUdi);

  static struct pci_driver pdr;
  static struct usb_driver udr;

  static struct  pci_device_id  mpPdi[]  = {
    { 0x16DA, 0x0011, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (kernel_ulong_t)&adPciXl  },
    { 0x10E8, 0x8507, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (kernel_ulong_t)&adPciAmcc  },
    { 0x10B5, 0x9050, 0x10B5, 0x1072, 0, 0, (kernel_ulong_t)&adPciPlx  },
    { 0, }
  };   




  int ieeeopen(struct inode * pin, struct file *fp) { return xxx_open(&dm, pin, fp); }
  //int ieeemmap(struct file*pFil, struct vm_area_struct *pVma) { return lumi_mmap( &dm, pFil, pVma); }

  static struct file_operations gpib_fops=
  {
    owner: THIS_MODULE, 
    open: ieeeopen,
    release: xxx_release,
    ioctl: xxx_ioctl,
    //mmap: ieeemmap,
  };



  /*
   * parameters for non-pnp adaptors
   * 
   * these are enumerated before any pnp-adaptor
   */
  #define MAX_ISA 4

  static int mpIoIsa[MAX_ISA];
  static int mpIrqIsa[MAX_ISA];
  int fDebug=0;
  static int cIsa;

 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
  MODULE_PARM(mpIoIsa,"1-" __MODULE_STRING(MAX_ISA) "i");
  MODULE_PARM(mpIrqIsa, "1-" __MODULE_STRING(MAX_ISA) "i");
  MODULE_PARM(fDebug,"i");
  module_param_array(mpIrqIsa, int, cIsa, 0444);
  module_param_array(mpIoIsa, int, cIsa, 0444);
 #else
  //MODULE_PARM_DESC(mpIoIsa,"1-" __MODULE_STRING(MAX_ISA) "i");
  //MODULE_PARM_DESC(mpIrqIsa, "1-" __MODULE_STRING(MAX_ISA) "i");
  //MODULE_PARM_DESC(fDebug,"i");
  module_param(fDebug, int, 0);
  module_param_array(mpIrqIsa, int, &cIsa, 0444);
  module_param_array(mpIoIsa, int, &cIsa, 0444);
 #endif



  struct LumiModStruct dm = {
    "igpib",
    "ines ieee488 GPIB",
    "6.120 (" __DATE__ ") ",
    &umGpib,
    &fDebug,
    &gpib_fops,
    MAX_ISA,
    mpIoIsa,
    mpIrqIsa,
    &adIsa,
    &pdr,
    &udr,
    "driver/igpib"
  };



  int pci_probe(struct pci_dev *pdv, const struct pci_device_id *pdi) { return pdm_pci_probe(&dm, pdv, pdi); }
  void pci_remove(struct pci_dev *pdv) { return pdm_pci_remove(&dm, pdv); }

  static struct pci_driver pdr = {
    name: 	"igpib",
    id_table:	mpPdi,
    probe:	pci_probe,
    remove:	pci_remove,
    /* operations set by lumi */
  };


  int usb_probe(struct usb_interface *pui, const struct usb_device_id *pdi) { return pdm_usb_probe(&dm, pui, pdi); }
  void usb_disconnect(struct usb_interface *pui) { return pdm_usb_disconnect(&dm, pui); }
  
  static struct usb_driver udr = {
   #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
    .owner = THIS_MODULE,
   #endif
    .name= 	"igpib",
    .probe= 	usb_probe,
    .disconnect=  usb_disconnect,
    .id_table=	mpUdi
  };


  static int init_gpib (void) { return pdm_init_module(&dm); }
  static void cleanup_gpib (void) { return pdm_cleanup_module(&dm); }

  module_init(init_gpib);
  module_exit(cleanup_gpib);

